@@ -111,6 +111,7 @@ nma_mobile_access_method_unref (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_name:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none): the name of the method.
  */
@@ -124,6 +125,7 @@ nma_mobile_access_method_get_name (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_username:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none): the username.
  */
@@ -137,6 +139,7 @@ nma_mobile_access_method_get_username (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_password:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none): the password.
  */
@@ -150,6 +153,7 @@ nma_mobile_access_method_get_password (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_gateway:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none): the gateway.
  */
@@ -163,6 +167,7 @@ nma_mobile_access_method_get_gateway (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_dns:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none) (array zero-terminated=1) (element-type utf8): the list of DNS.
  */
@@ -176,6 +181,7 @@ nma_mobile_access_method_get_dns (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_3gpp_apn:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: (transfer none): the 3GPP APN.
  */
@@ -189,6 +195,7 @@ nma_mobile_access_method_get_3gpp_apn (NMAMobileAccessMethod *method)
 
 /**
  * nma_mobile_access_method_get_family:
+ * @method: a #NMAMobileAccessMethod
  *
  * Returns: a #NMAMobileFamily.
  */
@@ -253,8 +260,7 @@ nma_mobile_provider_unref (NMAMobileProvider *provider)
 		g_free (provider->name);
 		g_hash_table_destroy (provider->lcl_names);
 
-		g_slist_foreach (provider->methods, (GFunc) nma_mobile_access_method_unref, NULL);
-		g_slist_free (provider->methods);
+		g_slist_free_full (provider->methods, (GDestroyNotify) nma_mobile_access_method_unref);
 
 		if (provider->mcc_mnc)
 			g_ptr_array_unref (provider->mcc_mnc);
@@ -268,6 +274,7 @@ nma_mobile_provider_unref (NMAMobileProvider *provider)
 
 /**
  * nma_mobile_provider_get_name:
+ * @provider: a #NMAMobileProvider
  *
  * Returns: (transfer none): the name of the provider.
  */
@@ -378,6 +385,7 @@ nma_country_info_unref (NMACountryInfo *country_info)
 
 /**
  * nma_country_info_get_country_code:
+ * @country_info: a #NMACountryInfo
  *
  * Returns: (transfer none): the code of the country.
  */
@@ -391,6 +399,7 @@ nma_country_info_get_country_code (NMACountryInfo *country_info)
 
 /**
  * nma_country_info_get_country_name:
+ * @country_info: a #NMACountryInfo
  *
  * Returns: (transfer none): the name of the country.
  */
@@ -404,6 +413,7 @@ nma_country_info_get_country_name (NMACountryInfo *country_info)
 
 /**
  * nma_country_info_get_providers:
+ * @country_info: a #NMACountryInfo
  *
  * Returns: (element-type NMAMobileProvider) (transfer none): the
  *	 list of #NMAMobileProvider this country exposes.
@@ -1134,7 +1144,7 @@ struct _NMAMobileProvidersDatabasePrivate {
  * @self: a #NMAMobileProvidersDatabase.
  *
  * Returns: (element-type utf8 NMACountryInfo) (transfer none): a
- *	 hash table where keys are country names #gchar and values are #NMACountryInfos.
+ *	 hash table where keys are country names #gchar and values are #NMACountryInfo.
  */
 GHashTable *
 nma_mobile_providers_database_get_countries (NMAMobileProvidersDatabase *self)
