@@ -129,7 +129,7 @@ eap_method_init (gsize obj_size,
                  EMFillConnectionFunc fill_connection,
                  EMUpdateSecretsFunc update_secrets,
                  EMDestroyFunc destroy,
-                 const char *ui_file,
+                 const char *ui_resource,
                  const char *ui_widget_name,
                  const char *default_field,
                  gboolean phase2)
@@ -138,7 +138,7 @@ eap_method_init (gsize obj_size,
 	GError *error = NULL;
 
 	g_return_val_if_fail (obj_size > 0, NULL);
-	g_return_val_if_fail (ui_file != NULL, NULL);
+	g_return_val_if_fail (ui_resource != NULL, NULL);
 	g_return_val_if_fail (ui_widget_name != NULL, NULL);
 
 	method = g_slice_alloc0 (obj_size);
@@ -154,9 +154,9 @@ eap_method_init (gsize obj_size,
 	method->phase2 = phase2;
 
 	method->builder = gtk_builder_new ();
-	if (!gtk_builder_add_from_file (method->builder, ui_file, &error)) {
-		g_warning ("Couldn't load UI builder file %s: %s",
-		           ui_file, error->message);
+	if (!gtk_builder_add_from_resource (method->builder, ui_resource, &error)) {
+		g_warning ("Couldn't load UI builder resource %s: %s",
+		           ui_resource, error->message);
 		eap_method_unref (method);
 		return NULL;
 	}
@@ -164,7 +164,7 @@ eap_method_init (gsize obj_size,
 	method->ui_widget = GTK_WIDGET (gtk_builder_get_object (method->builder, ui_widget_name));
 	if (!method->ui_widget) {
 		g_warning ("Couldn't load UI widget '%s' from UI file %s",
-		           ui_widget_name, ui_file);
+		           ui_widget_name, ui_resource);
 		eap_method_unref (method);
 		return NULL;
 	}
@@ -215,14 +215,9 @@ eap_method_validate_filepicker (GtkBuilder *builder,
                                 GError **error)
 {
 	GtkWidget *widget;
-	char *filename;
+	gs_free char *filename = NULL;
 	NMSetting8021x *setting;
-	gboolean success = TRUE;
-
-	if (item_type == TYPE_PRIVATE_KEY) {
-		if (!password || *password == '\0')
-			success = FALSE;
-	}
+	gboolean success;
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, name));
 	g_assert (widget);
@@ -231,12 +226,14 @@ eap_method_validate_filepicker (GtkBuilder *builder,
 		if (item_type != TYPE_CA_CERT) {
 			success = FALSE;
 			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("no file selected"));
-		}
+		} else
+			success = TRUE;
 		goto out;
 	}
 
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
 		success = FALSE;
+		g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("file “%s” does not exist"), filename);
 		goto out;
 	}
 
@@ -258,8 +255,6 @@ eap_method_validate_filepicker (GtkBuilder *builder,
 	g_object_unref (setting);
 
 out:
-	g_free (filename);
-
 	if (!success && error && !*error)
 		g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("unspecified error validating eap-method file"));
 
@@ -270,31 +265,6 @@ out:
 	return success;
 }
 
-#ifdef LIBNM_GLIB_BUILD
-static const char *
-find_tag (const char *tag, const char *buf, gsize len)
-{
-	gsize i, taglen;
-
-	taglen = strlen (tag);
-	if (len < taglen)
-		return NULL;
-
-	for (i = 0; i < len - taglen + 1; i++) {
-		if (memcmp (buf + i, tag, taglen) == 0)
-			return buf + i;
-	}
-	return NULL;
-}
-
-static const char *pem_rsa_key_begin = "-----BEGIN RSA PRIVATE KEY-----";
-static const char *pem_dsa_key_begin = "-----BEGIN DSA PRIVATE KEY-----";
-static const char *pem_pkcs8_enc_key_begin = "-----BEGIN ENCRYPTED PRIVATE KEY-----";
-static const char *pem_pkcs8_dec_key_begin = "-----BEGIN PRIVATE KEY-----";
-static const char *pem_cert_begin = "-----BEGIN CERTIFICATE-----";
-static const char *proc_type_tag = "Proc-Type: 4,ENCRYPTED";
-static const char *dek_info_tag = "DEK-Info:";
-
 static gboolean
 file_has_extension (const char *filename, const char *extensions[])
 {
@@ -320,6 +290,31 @@ file_has_extension (const char *filename, const char *extensions[])
 	return found;
 }
 
+#if !LIBNM_BUILD
+static const char *
+find_tag (const char *tag, const char *buf, gsize len)
+{
+	gsize i, taglen;
+
+	taglen = strlen (tag);
+	if (len < taglen)
+		return NULL;
+
+	for (i = 0; i < len - taglen + 1; i++) {
+		if (memcmp (buf + i, tag, taglen) == 0)
+			return buf + i;
+	}
+	return NULL;
+}
+
+static const char *pem_rsa_key_begin = "-----BEGIN RSA PRIVATE KEY-----";
+static const char *pem_dsa_key_begin = "-----BEGIN DSA PRIVATE KEY-----";
+static const char *pem_pkcs8_enc_key_begin = "-----BEGIN ENCRYPTED PRIVATE KEY-----";
+static const char *pem_pkcs8_dec_key_begin = "-----BEGIN PRIVATE KEY-----";
+static const char *pem_cert_begin = "-----BEGIN CERTIFICATE-----";
+static const char *proc_type_tag = "Proc-Type: 4,ENCRYPTED";
+static const char *dek_info_tag = "DEK-Info:";
+
 static gboolean
 pem_file_is_encrypted (const char *buffer, gsize bytes_read)
 {
@@ -403,57 +398,28 @@ out:
 static gboolean
 default_filter_privkey (const GtkFileFilterInfo *filter_info, gpointer user_data)
 {
-#ifdef LIBNM_GLIB_BUILD
 	const char *extensions[] = { ".der", ".pem", ".p12", ".key", NULL };
-#endif
-	gboolean require_encrypted = !!user_data;
-	gboolean is_encrypted;
 
 	if (!filter_info->filename)
 		return FALSE;
 
-#if defined (LIBNM_GLIB_BUILD)
 	if (!file_has_extension (filter_info->filename, extensions))
 		return FALSE;
 
-	is_encrypted = TRUE;
-	if (   !file_is_der_or_pem (filter_info->filename, TRUE, &is_encrypted)
-	    && !nm_utils_file_is_pkcs12 (filter_info->filename))
-		return FALSE;
-#elif defined (LIBNM_BUILD)
-	is_encrypted = FALSE;
-	if (!nm_utils_file_is_private_key (filter_info->filename, &is_encrypted))
-		return FALSE;
-#else
-#error neither LIBNM_BUILD nor LIBNM_GLIB_BUILD defined
-#endif
-
-	return require_encrypted ? is_encrypted : TRUE;
+	return TRUE;
 }
 
 static gboolean
 default_filter_cert (const GtkFileFilterInfo *filter_info, gpointer user_data)
 {
-#ifdef LIBNM_GLIB_BUILD
 	const char *extensions[] = { ".der", ".pem", ".crt", ".cer", NULL };
-#endif
 
 	if (!filter_info->filename)
 		return FALSE;
 
-#if defined (LIBNM_GLIB_BUILD)
 	if (!file_has_extension (filter_info->filename, extensions))
 		return FALSE;
 
-	if (!file_is_der_or_pem (filter_info->filename, FALSE, NULL))
-		return FALSE;
-#elif defined (LIBNM_BUILD)
-	if (!nm_utils_file_is_certificate (filter_info->filename))
-		return FALSE;
-#else
-#error neither LIBNM_BUILD nor LIBNM_GLIB_BUILD defined
-#endif
-
 	return TRUE;
 }
 
@@ -477,8 +443,22 @@ gboolean
 eap_method_is_encrypted_private_key (const char *path)
 {
 	GtkFileFilterInfo info = { .filename = path };
+	gboolean is_encrypted;
 
-	return default_filter_privkey (&info, (gpointer) TRUE);
+	if (!default_filter_privkey (&info, NULL))
+		return FALSE;
+
+#if LIBNM_BUILD
+	is_encrypted = FALSE;
+	if (!nm_utils_file_is_private_key (path, &is_encrypted))
+		return FALSE;
+#else
+	is_encrypted = TRUE;
+	if (   !file_is_der_or_pem (path, TRUE, &is_encrypted)
+	    && !nm_utils_file_is_pkcs12 (path))
+		return FALSE;
+#endif
+	return is_encrypted;
 }
 
 /* Some methods (PEAP, TLS, TTLS) require a CA certificate. The user can choose
